#include <Ticker.h>
#include <ESP8266WiFi.h>
#include <PubSubClient.h>
#include <Wire.h>
#include "Gsender.h"



Ticker timer; //declaration du timer

bool tempo=LOW;



/**
 * wifi
 */
#pragma region Globals
const char* ssid = "";                           // WIFI network name
const char* password = "";                       // WIFI network password
const char* destinataire = "";                   //adresse mail destinataire
uint8_t connection_state = 0;                    // Connected to WIFI or not
uint16_t reconnect_interval = 10000;             // If not connected wait time to try again
#pragma endregion Globals

/**
 * mqtt
 */
const char* mqtt_server = "debian";  //nom de la machine qui éberge le server MQTT 

#define moisture_topic "sensor/moisture" //Topic humidité
#define lum_topic "sensor/lum" //Topic luminosité

WiFiClient espClient;
PubSubClient client(espClient);


/**
 * Pins
 */
//const byte temperature = A0;
const byte ledPin = 2;
const byte reset = 12;



/**
 * Connection Wifi
 */


uint8_t WiFiConnect(const char* nSSID = nullptr, const char* nPassword = nullptr)
{
    static uint16_t attempt = 0;
    Serial.print("Connecting to ");
    if(nSSID) {
        WiFi.begin(nSSID, nPassword);  
        Serial.println(nSSID);
    } else {
        WiFi.begin(ssid, password);
        Serial.println(ssid);
    }

    uint8_t i = 0;
    while(WiFi.status()!= WL_CONNECTED && i++ < 50)
    {
        delay(200);
        Serial.print(".");
    }
    ++attempt;
    Serial.println("");
    if(i == 51) {
        Serial.print("Connection: TIMEOUT on attempt: ");
        Serial.println(attempt);
        if(attempt % 2 == 0)
            Serial.println("Check if access point available or SSID and Password\r\n");
        return false;
    }
    Serial.println("Connection: ESTABLISHED");
    Serial.print("Got IP address: ");
    Serial.println(WiFi.localIP());
    return true;
}

void Awaits()
{
    uint32_t ts = millis();
    while(!connection_state)
    {
        delay(50);
        if(millis() > (ts + reconnect_interval) && !connection_state){
            connection_state = WiFiConnect();
            ts = millis();
        }
    }
}




/**
 * Interruption
 */

void interrupt()
{
   tempo = HIGH; 
   
}
 



/**
 * Initialisation
 */

void setup()
{

    Wire.begin(14,13); //pin 14 = SDA, pin 13 = SCL
    
    Serial.begin(115200);
    Serial.println("debut");
    connection_state = WiFiConnect();
    if(!connection_state)  // if not connected to WIFI
        Awaits();          // constantly trying to connect

   

    client.setServer(mqtt_server, 1883);

    pinMode(ledPin, OUTPUT);
    pinMode(reset, OUTPUT);

    timer.attach(60,interrupt); //toutes les 60 secondes on lance la fonction interrupt
}


/**
 * Écriture registre chirp
 */
void writeI2CRegister8bit(int addr, int value)
{
  Wire.beginTransmission(addr);
  Wire.write(value);
  Wire.endTransmission();
}


/**
 * Lecture d'un registre du capteur
 */
unsigned int readI2CRegister16bit(int addr, int reg) 
{
  Wire.beginTransmission(addr);
  Wire.write(reg);
  Wire.endTransmission();
  delay(1100);
  Wire.requestFrom(addr, 2);
  unsigned int t = Wire.read() << 8;
  t = t | Wire.read();
  return t;
}

 /**
  * Connection et re-connection au serveur MQTT
  */
void reconnect() {
 // Tentative de connection
 if (!client.connected()) {
 Serial.print("Attempting MQTT connection...");
 // Attempt to connect
 if (client.connect("ESP8266 Client")) {
  Serial.println("connected");
  // ... and subscribe to topic "ledStatus"
  client.subscribe("ledStatus");
 } else {
  Serial.print("failed, state=");
  Serial.println(client.state());
  }
 }
}



/**
 * Lecture du capteur d'humidité et reset en cas de mauvaise valeur
 */
 unsigned int getMoisture(){  
  int moistureVal = readI2CRegister16bit(0x20, 0);
  while(moistureVal == -1)
  {
    delay(500);
    Serial.println("Reset chirp...");
    // Chirp ne répond pas, reset
    digitalWrite(reset, LOW); 
    delay(500);
    digitalWrite(reset, HIGH); 
    delay(500);
    moistureVal = readI2CRegister16bit(0x20, 0);
  }

  while(moistureVal == 65535){
    moistureVal = readI2CRegister16bit(0x20, 0);
  }
 
  return moistureVal;
}



int humid = 0; // variable qui contient la valeur de l'humidité
int lectureLum = 0; // variable qui contient la valeur de la luminosité
int prec = 0;//temposisation

void loop()
{  

  
 
 /**
  * Connection ou re-connection au serveur MQTT
  */ 
  if (!client.connected()) 
  {
   reconnect();
  }
  //client.loop();

  /**
   * Requète de mesure de luminosité
   */
  writeI2CRegister8bit(0x20, 3);
  delay(500);
 
 /**
  * Lecture humidité 
  */
  humid = getMoisture();
  Serial.print("Moisture : ");
  Serial.println(humid);
  delay(500);
  
/**
 * Lecture luminosité
 */
  prec=lectureLum;//on stock la valeur précédente
  lectureLum=readI2CRegister16bit(0x20, 4);
  lectureLum = 1600 - lectureLum;
  Serial.print( "Prec : ");
  Serial.print(prec);
  Serial.print( " Luminosité : ");
  Serial.println(lectureLum);




  /**
   * Publication MQTT
   */
  if(client.connected())
  {
    client.publish(moisture_topic, String(humid).c_str(), true);
    client.publish(lum_topic, String(lectureLum).c_str(), true);
  }


  /**
   * Envoi du mail sur l'humidité
   * 
   */


  if (humid<300 && tempo==HIGH)
  {    
     Gsender *gsender = Gsender::Instance();    // Getting pointer to class instance
     String subject = "Eau plante";
     if(gsender->Subject(subject)->Send(destinataire, "Coucou ! :) Je suis votre plante et j'ai un peu soif :'(")) {
         Serial.println("Message send.");
     } else {
         Serial.print("Error sending message: ");
         Serial.println(gsender->getError());
     }
     tempo = LOW;
   } else { tempo = LOW; }
  



  /**
   * Envoi du mail sur la luminosité si front descendant en dessous du seuil
   * 
   */

 
  if (lectureLum<600 && prec>600)
  {    
     Gsender *gsender = Gsender::Instance();    // Getting pointer to class instance
     String subject = "Lumiere Plante";
     if(gsender->Subject(subject)->Send(destinataire, "Coucou ! C'est moi votre petite plante :) J'aurais besoin d'un peu de lumière s'il vous plait :)")) {
         Serial.println("Message send.");
     } else {
         Serial.print("Error sending message: ");
         Serial.println(gsender->getError());
     }
   }
   
  delay(500);
}
